@@ -16,17 +16,17 @@ class Pipe:
         self._filters = []
 
     def model(self, model_id):
-        new_pipe = copy.copy(self)
+        new_pipe = copy.deepcopy(self)
         new_pipe._model = model_id
         return new_pipe
 
     def object(self, obj):
-        new_pipe = copy.copy(self)
+        new_pipe = copy.deepcopy(self)
         new_pipe._object = obj
         return new_pipe
 
     def filter(self, path: str, operator: str, value: any):
-        new_pipe = copy.copy(self)
+        new_pipe = copy.deepcopy(self)
         new_pipe._filters.append({
             'path': path,
             'operator': operator,
@@ -34,8 +34,17 @@ class Pipe:
         })
         return new_pipe
 
+    def equal(self, path: str, value: any):
+        return self.filter(path, 'eq', value)
+
+    def greater_than(self, path: str, value: any):
+        return self.filter(path, 'gt', value)
+
+    def less_than(self, path: str, value: any):
+        return self.filter(path, 'lt', value)
+
     def order(self, path: str, asc=True):
-        new_pipe = copy.copy(self)
+        new_pipe = copy.deepcopy(self)
         new_pipe._orders.append({
             'path': path,
             'ascending': asc,
@@ -130,55 +139,7 @@ class Pipe:
             return None
 
     def _path_source(self, path: str):
-        path_node = self._model_node
-        source_node = None
-        cur_node = {}
-        format = ''
-        paths = path.split('.')
-
-        if len(paths) > 0 and paths[0] != '':
-            paths.append('')
-
-        def descend_path(node, node_ptr):
-            nonlocal source_node, cur_node
-            if not source_node:
-                source_node = node
-                cur_node = node_ptr
-            else:
-                cur_node = node
-                cur_node = node_ptr
-
-        for p in paths:
-            if p != '':
-                if path_node['type'] == 'map':
-                    for e in path_node['map']['entries']:
-                        if e['name'] == p:
-                            path_node = e['node']
-                            break
-                    else:
-                        raise RuntimeError(f'Entry not found: {p}')
-                    continue
-
-            while True:
-                path_node_type = path_node['type']
-
-                if path_node_type == 'value':
-                    node = copy.copy(path_node)
-                    format = path_node['value']['format']
-                    descend_path(node, None)
-                    break
-
-                if path_node_type == 'relation':
-                    node = copy.copy(path_node)
-                    path_node = path_node['relation']['node']
-                    descend_path(node, node['relation']['node'])
-                    continue
-
-                if path_node_type == 'specials':
-                    node = copy.copy(path_node)
-                    path_node = path_node['specials']['node']
-                    descend_path(node, node['specials']['node'])
-                    continue
+        source_node, format = _cut_map_nodes(path.split('.'), self._model_node)
 
         return {
             'type': 'node',
@@ -211,3 +172,35 @@ class Pipe:
         new_pipe._model_node = copy.deepcopy(self._model_node)
         new_pipe._node_type = self._node_type
         return new_pipe
+
+
+def _cut_map_nodes(path: list[str], iter_node: any) -> [any, str]:
+    iter_type = iter_node['type']
+
+    if iter_type == 'map':
+        if len(path) == 0:
+            raise RuntimeError('Path leads to a map. Add a field by attaching it with a dot (.)')
+
+        for e in iter_node['map']['entries']:
+            if e['name'] == path[0]:
+                return _cut_map_nodes(path[1:], e['node'])
+        raise RuntimeError(f'Entry not found: {path[0]}')
+
+    elif iter_type == 'value':
+        if len(path) != 0:
+            raise RuntimeError(f'Path exceeds the node structure. Leftover path: ...{".".join(path)}')
+        return copy.deepcopy(iter_node), iter_node['value']['format']
+
+    elif iter_type == 'relation':
+        return_node, format = _cut_map_nodes(path, iter_node['relation']['node'])
+        iter_node = copy.deepcopy(iter_node)
+        iter_node['relation']['node'] = return_node
+        return iter_node, format
+
+    elif iter_type == 'specials':
+        return_node, format = _cut_map_nodes(path, iter_node['specials']['node'])
+        iter_node = copy.deepcopy(iter_node)
+        iter_node['specials']['node'] = return_node
+        return iter_node, format
+
+    raise RuntimeError(f'Unsupported node type: {iter_type}')
