@@ -57,10 +57,9 @@ def test_client__load_universe():
 
     print()
 
-    pipe = Pipe('$getCrawljob').filter('running', 'eq', False)
-    crawljobs = client.get_objects(pipe)
-    for cj in crawljobs:
-        print(cj)
+    pipe = Pipe('$putCrawljob')
+    crawljob = client.put_object(pipe, {'running': True})
+    print(crawljob)
 
     # items = get_item_pipe.model('item').filter('itemId', 'eq', '7743023').get_objects()
     # print(items)
